// SerialID: [e4a22a75-a938-4302-8b9a-d405c01db428]
using System;
using System.Collections.Generic;
using UnityEngine.UI;
using UnityEngine;

public class AgentExecutor : MonoBehaviour
{
    [SerializeField] private List<AgentItem> agents = null;
    private List<AgentItem> Agents => agents;
    private List<float> scores=new List<float>(){0,0};

    [Header("UI References"), SerializeField] private Text populationText = null;
    [Header("name1 UI"), SerializeField] private Text name1Text = null;
    [Header("name2 UI"), SerializeField] private Text name2Text = null;
    [Header("win1 UI"), SerializeField] private Text win1Text = null;
    [Header("win2 UI"), SerializeField] private Text win2Text = null;
    [Header("time1 UI"), SerializeField] private Text time1Text = null;
    [Header("time2 UI"), SerializeField] private Text time2Text = null;

    private Text PopulationText { get { return populationText; } }
    private List<float> lastScores = new List<float>();
    private bool alreadyGoal=false;
    private bool time1fix=false;
    private bool time2fix=false;


    private void Start() {
        if(Agents == null || Agents.Count == 0) {
            return;
        }
        name1Text.text=Agents[0].name;
        name2Text.text=Agents[1].name;
        win1Text.enabled=false;
        win2Text.enabled=false;

        /*
            <変更ポイント>
            自分のオリジナルのAIを用いたい場合はこうする
            Agents[0/1].brain = QBrain.Load(Agents[0/1].learning);
            Agents[0/1].run=RunX;

            デフォルトではどちらもニューロエボリューションに基づくAgentが設定されている

        */

        //いじる
        Agents[0].brain = XBrain.Load(Agents[0].learning);
        Agents[0].run=RunX;
        Agents[1].brain = XBrain.Load(Agents[1].learning);
        Agents[1].run=RunX;

        //いじらない
        Agents[0].agent.isBattle=true;
        Agents[0].agent.agentExecutor=this;
        Agents[0].agent.agentIndex=0;
        Agents[1].agent.isBattle=true;
        Agents[1].agent.agentExecutor=this;
        Agents[1].agent.agentIndex=1;
    }

    private void FixedUpdate() {
        ExecuteAction(Agents[0],0);
        ExecuteAction(Agents[1],1);
        UpdateText();
    }

    private void ExecuteAction(AgentItem item,int index){
        if(item.agent.IsDone) {
            return;
        }

        item.run(item);
        scores[index]=item.agent.GetDistance();
    }

    private void RunX(AgentItem item){
        /*
            自分のオリジナルの学習規則に基づいてAgentを動かす関数。
            CarAgent.csのOriginalObservation関数とXBrain.csのGetAction関数を基本的には編集すれば良い(必要に応じて新しい関数を追加したりする)
        */
        var observation = item.agent.OriginalObservations();
        var action = (item.brain as XBrain).GetAction(observation);
        item.agent.AgentAction(action);
    } 

    private void RunQ(AgentItem item) {
        var observation = item.agent.GetState();
        int actionNo = (item.brain as QBrain).GetAction(observation);
        var action = item.agent.ActionNumberToVectorAction(actionNo);
        item.agent.AgentAction(action);
    }

    private void RunNE(AgentItem item) {
        var observation = item.agent.CollectObservations();
        item.agent.AgentAction((item.brain as NNBrain).GetAction(observation));
    }

    private void UpdateText() {
        string text="";
        for (int i = 0; i < scores.Count; i++)
        {
            text+=Agents[i].name+" Car"+(i+1).ToString()+" Score: "+scores[i].ToString()+"\n";
        }
        PopulationText.text = text;
        if(time1fix==false){
            time1Text.text="time:"+Time.time.ToString();
        }
        if(time2fix==false){
            time2Text.text="time:"+Time.time.ToString();
        }
        
    }

    public void Win(int agentIndex){
        if(alreadyGoal==false){
            if(agentIndex==0){
                win1Text.enabled=true;
            }else{
                win2Text.enabled=true;
            }
        }
        if(agentIndex==0){
            time1fix=true;
        }else{
            time2fix=true;
        }
        alreadyGoal=true;
    }

    [Serializable]
    public class AgentItem
    {
        public Brain brain;
        public Agent agent;
        public TextAsset learning;
        public Action<AgentItem> run;
        public string name;
    }
}
